@@ -1,6 +1,6 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2009, 2010, 2014, 2016 Laboratoire de Recherche et
-// Developpement de l'Epita
+// Copyright (C) 2009, 2010, 2014, 2016, 2018 Laboratoire de Recherche
+// et Developpement de l'Epita
 //
 // This file is part of Spot, a model checking library.
 //
@@ -17,6 +17,7 @@
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+#include "config.h"
 #include <spot/kripke/fairkripke.hh>
 
 namespace spot
