@@ -241,6 +241,7 @@ namespace spot
         C(Fusion);
         C(Star);
         C(FStar);
+        C(first_match);
 #undef C
       }
     SPOT_UNREACHABLE();
@@ -253,46 +254,35 @@ namespace spot
     //
     // When we construct a formula such as Multop(Op,X,Multop(Op,Y,Z))
     // we will want to inline it as Multop(Op,X,Y,Z).
-    {
-      vec inlined;
-      vec::iterator i = v.begin();
-      while (i != v.end())
-        {
-          // Some simplification routines erase terms using null
-          // pointers that we must ignore.
-          if ((*i) == nullptr)
-            {
-              // FIXME: For commutative operators we should replace
-              // the pointer by the first non-null value at the end
-              // of the array instead of calling erase.
-              i = v.erase(i);
+    //
+    // At the same time, it's possible that vec contains some null
+    // pointers we should remove.  We can do it in the same loop.
+    //
+    // It is simpler to construct a separate vector to do that, but that's
+    // only needed if we have nested multops or null poiners.
+    if (std::find_if(v.begin(), v.end(),
+                     [o](const fnode* f) { return f == nullptr || f->is(o); })
+        != v.end())
+      {
+        vec inlined;
+        for (const fnode* f: v)
+          {
+            if (f == nullptr)
               continue;
-            }
-          if ((*i)->is(o))
-            {
-              unsigned ps = (*i)->size();
-              for (unsigned n = 0; n < ps; ++n)
-                inlined.emplace_back((*i)->nth(n)->clone());
-              (*i)->destroy();
-              // FIXME: Do not use erase.  See previous FIXME.
-              i = v.erase(i);
-              continue;
-            }
-          // All operators except "Concat" and "Fusion" are
-          // commutative, so we just keep a list of the inlined
-          // arguments that should later be added to the vector.
-          // For concat we have to keep track of the order of
-          // all the arguments.
-          if (o == op::Concat || o == op::Fusion)
-            inlined.emplace_back(*i);
-          ++i;
-        }
-      if (o == op::Concat || o == op::Fusion)
+            if (f->is(o))
+              {
+                unsigned ps = f->size();
+                for (unsigned n = 0; n < ps; ++n)
+                  inlined.emplace_back(f->nth(n)->clone());
+                f->destroy();
+              }
+            else
+              {
+                inlined.emplace_back(f);
+              }
+          }
         v.swap(inlined);
-      else
-        v.insert(v.end(), inlined.begin(), inlined.end());
-    }
-
+      }
     if (o != op::Concat && o != op::Fusion)
       std::sort(v.begin(), v.end(), formula_ptr_less_than_bool_first());
 
@@ -826,6 +816,18 @@ namespace spot
         if (f->is_boolean())
           return unop(op::Not, f);
         break;
+      case op::first_match:
+        // first_match(first_match(sere)) = first_match(sere);
+        // first_match(b) = b
+        if (f->is(o) || f->is_boolean())
+          return f;
+        // first_match(r*) = [*0]
+        if (f->accepts_eword())
+          {
+            f->destroy();
+            return eword();
+          }
+        break;
       default:
         SPOT_UNREACHABLE();
       }
@@ -1617,6 +1619,23 @@ namespace spot
             }
         }
         break;
+      case op::first_match:
+        props = children[0]->props;
+        assert(is_.sere_formula);
+        // {(a[+];b*);c*}<>->d is stutter invariant
+        // {first_match(a[+];b*);c*}<>->d is NOT.
+        is_.syntactic_si = false;
+        is_.boolean = false;
+        is_.ltl_formula = false;
+        is_.psl_formula = false;
+        is_.eventual = false;
+        is_.universal = false;
+        is_.syntactic_safety = false;
+        is_.syntactic_guarantee = false;
+        is_.syntactic_obligation = false;
+        is_.syntactic_recurrence = false;
+        is_.syntactic_persistence = false;
+        break;
       }
   }
 
@@ -1627,11 +1646,14 @@ namespace spot
     const fnode* res = f;
     if (max < min)
       std::swap(min, max);
-    for (unsigned i = min; i < max; ++i)
-      {
-        const fnode* a = f->clone();
-        res = fnode::multop(bo, {a, fnode::unop(uo, res)});
-      }
+    if (max != unbounded())
+      for (unsigned i = min; i < max; ++i)
+        {
+          const fnode* a = f->clone();
+          res = fnode::multop(bo, {a, fnode::unop(uo, res)});
+        }
+    else
+      res = fnode::unop(bo == op::Or ? op::F : op::G, res);
     for (unsigned i = 0; i < min; ++i)
       res = fnode::unop(uo, res);
     return res;
@@ -1714,7 +1736,7 @@ namespace spot
         {
           if (!cnt++)
             std::cerr << "*** m.uniq is not empty ***\n";
-          i->dump(std::cerr) << std::endl;
+          i->dump(std::cerr) << '\n';
         }
     return cnt == 0;
   }
@@ -1743,6 +1765,43 @@ namespace spot
     return Concat({Star(Concat({s, b}), min, max), s});
   }
 
+  formula formula::sugar_delay(const formula& b,
+                               unsigned min, unsigned max)
+  {
+    // ##[min:max] b = 1[*min:max];b
+    return Concat({Star(tt(), min, max), b});
+  }
+  formula formula::sugar_delay(const formula& a, const formula& b,
+                               unsigned min, unsigned max)
+  {
+    // If min>=1
+    //   a ##[min:max] b  = a;1[*min-1:max-1];b
+    // If min==0 we can use
+    //   a ##[0:0] b = a:b
+    //   a ##[0:max] b = a:(1[*0:max];b)  if a rejects [*0]
+    //   a ##[0:max] b = (a;1[*0:max]):b  if b rejects [*0]
+    //   a ##[0:max] b = (a:b)|(a;[*0:max-1];b)  else
+    if (min > 0)
+      {
+        --min;
+        if (max != unbounded())
+          --max;
+        return Concat({a, Star(tt(), min, max), b});
+      }
+    if (max == 0)
+      return Fusion({a, b});
+    if (!a.accepts_eword())
+      return Fusion({a, Concat({Star(tt(), 0, max), b})});
+    if (!b.accepts_eword())
+      return Fusion({Concat({a, Star(tt(), 0, max)}), b});
+
+    if (max != unbounded())
+      --max;
+    formula left = Fusion({a, b});
+    formula right = Concat({a, Star(tt(), 0, max), b});
+    return OrRat({left, right});
+  }
+
   int atomic_prop_cmp(const fnode* f, const fnode* g)
   {
     return strverscmp(f->ap_name().c_str(), g->ap_name().c_str());
