@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2014-2018 Laboratoire de Recherche et Développement
+// Copyright (C) 2014-2019 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -23,11 +23,11 @@
 #include <algorithm>
 #include <queue>
 #include <spot/twa/bddprint.hh>
+#include <spot/twaalgos/bfssteps.hh>
 #include <spot/twaalgos/mask.hh>
 #include <spot/twaalgos/genem.hh>
 #include <spot/misc/escape.hh>
 
-
 namespace spot
 {
   void scc_info::report_need_track_states()
@@ -48,6 +48,13 @@ namespace spot
       ("scc_info was run with option STOP_ON_ACC");
   }
 
+  // this one is not yet needed in the hh file
+  static void report_need_stop_on_acc()
+  {
+    throw std::runtime_error
+      ("scc_info was not run with option STOP_ON_ACC");
+  }
+
   namespace
   {
     struct scc
@@ -68,6 +75,13 @@ namespace spot
     };
   }
 
+  scc_info::scc_info(const scc_and_mark_filter& filt, scc_info_options options)
+    : scc_info(filt.get_aut(), filt.start_state(),
+               filt.get_filter(),
+               const_cast<scc_and_mark_filter*>(&filt), options)
+  {
+  }
+
   scc_info::scc_info(const_twa_graph_ptr aut,
                      unsigned initial_state,
                      edge_filter filter,
@@ -461,6 +475,178 @@ namespace spot
       determine_usefulness();
   }
 
+  // A reimplementation of spot::bfs_steps for explicit automata.
+  // bool filter(const twa_graph::edge_storage_t&) returns true if the
+  // transition has to be filtered out.
+  // bool match(const twa_graph::edge_storage_t&) returns true if the BFS
+  // has to stop after this transition.
+  // Returns the destination of the matched transition, or -1 if no match has
+  // been found.
+  template <typename edge_filter_type,
+            typename step_matcher_type>
+  static int explicit_bfs_steps(const const_twa_graph_ptr aut, unsigned start,
+                                twa_run::steps& steps,
+                                edge_filter_type filter,
+                                step_matcher_type match)
+  {
+    auto& gr = aut->get_graph();
+    // The backlink of each state is the index of the edge that
+    // discovered it in the BFS, so BACKLINKS effectively describes a
+    // tree rooted at START.
+    std::vector<unsigned> backlinks(aut->num_states(), 0);
+    std::deque<unsigned> bfs_queue;
+    bfs_queue.emplace_back(start);
+    while (!bfs_queue.empty())
+      {
+        unsigned src = bfs_queue.front();
+        bfs_queue.pop_front();
+        for (auto& t: aut->out(src))
+          {
+            if (filter(t))
+              continue;
+
+            if (match(t))
+              {
+                // Build the path from START to T.DST by following the
+                // backlinks, starting at the end.
+                twa_run::steps path;
+                path.emplace_front(aut->state_from_number(t.src),
+                                   t.cond, t.acc);
+                unsigned src = t.src;
+                while (src != start)
+                  {
+                    unsigned bl_num = backlinks[src];
+                    assert(bl_num);
+                    auto& bl_edge = gr.edge_storage(bl_num);
+                    src = bl_edge.src;
+                    path.emplace_front(aut->state_from_number(src),
+                                       bl_edge.cond, bl_edge.acc);
+                  }
+                steps.splice(steps.end(), path);
+                return t.dst;
+              }
+
+            if (!backlinks[t.dst])
+              {
+                backlinks[t.dst] = aut->edge_number(t);
+                bfs_queue.push_back(t.dst);
+              }
+          }
+      }
+    return -1;
+  }
+
+  void scc_info::get_accepting_run(unsigned scc, twa_run_ptr r) const
+  {
+    const scc_info::scc_node& node = node_[scc];
+    if (!node.is_accepting())
+      throw std::runtime_error("scc_info::get_accepting_cycle needs to be "
+                               "called on an accepting scc");
+    if (SPOT_UNLIKELY(!(options_ & scc_info_options::STOP_ON_ACC)))
+      report_need_stop_on_acc();
+
+    unsigned init = aut_->get_init_state_number();
+
+    // The accepting cycle should honor any edge filter we have.
+    auto filter = [this](const twa_graph::edge_storage_t& t)
+                  {
+                    if (!filter_)
+                      return false;
+                    // Filter out ignored and cut transitions.
+                    return filter_(t, t.dst, filter_data_)
+                      != edge_filter_choice::keep;
+                  };
+
+    // The SCC exploration has a small optimization that can flag SCCs
+    // has accepting if they contain accepting self-loops, even if the
+    // SCC itself has some Fin acceptance to check.  So we have to
+    // deal with this situation before we look for the more complex
+    // case of satisfying the condition with a larger cycle.  We do
+    // this first, because it's good to return a small cycle if we
+    // can.
+    const acc_cond& acccond = aut_->acc();
+    for (unsigned s: node.states())
+      for (auto& e: aut_->out(s))
+        if (e.src == e.dst && !filter(e) && acccond.accepting(e.acc))
+          {
+            // We have found an accepting self-loop.  That's the cycle
+            // part of our accepting run.
+            r->cycle.clear();
+            r->cycle.emplace_front(aut_->state_from_number(e.src),
+                                   e.cond, e.acc);
+            // Add the prefix.
+            r->prefix.clear();
+            if (e.src != init)
+              explicit_bfs_steps(aut_, init, r->prefix,
+                                 [](const twa_graph::edge_storage_t&)
+                                 {
+                                   return false; // Do not filter.
+                                 },
+                                 [&](const twa_graph::edge_storage_t& t)
+                                 {
+                                   return t.dst == e.src;
+                                 });
+            return;
+          }
+
+
+    // Prefix search
+
+    r->prefix.clear();
+    int substart;
+    if (scc_of(init) == scc)
+      {
+        // The initial state is in the target SCC: no prefix needed.
+        substart = init;
+      }
+    else
+      {
+        substart = explicit_bfs_steps(aut_, init, r->prefix,
+            [](const twa_graph::edge_storage_t&)
+            {
+              return false; // Do not filter.
+            },
+            [&](const twa_graph::edge_storage_t& t)
+            {
+              // Match any state in the SCC.
+              return scc_of(t.dst) == scc;
+            });
+      }
+
+    const unsigned start = (unsigned)substart;
+
+    // Cycle search
+    acc_cond actual_cond = acccond.restrict_to(node.acc_marks())
+      .force_inf(node.acc_marks());
+    assert(!actual_cond.uses_fin_acceptance());
+    assert(!actual_cond.is_f());
+    acc_cond::mark_t acc_to_see = actual_cond.accepting_sets(node.acc_marks());
+    r->cycle.clear();
+
+    do
+      {
+        substart = explicit_bfs_steps(aut_, substart, r->cycle,
+            [&](const twa_graph::edge_storage_t& t)
+            {
+              // Stay in the specified SCC.
+              return scc_of(t.dst) != scc || filter(t);
+            },
+            [&](const twa_graph::edge_storage_t& t)
+            {
+              if (!acc_to_see) // We have seen all the marks, go back to start.
+                return t.dst == start;
+              if (t.acc & acc_to_see)
+                {
+                  acc_to_see -= t.acc;
+                  return true;
+                }
+              return false;
+            });
+        assert(0 <= substart);
+      }
+    while (acc_to_see || (unsigned)substart != start);
+  }
+
   std::ostream&
   dump_scc_info_dot(std::ostream& out,
                     const_twa_graph_ptr aut, scc_info* sccinfo)
@@ -537,6 +723,20 @@ namespace spot
                                              unsigned dst)
                              {
                                cur[src] = seen[src] = true;
+                               if (filter_)
+                                 {
+                                   twa_graph::edge_storage_t e;
+                                   e.cond = cond;
+                                   e.acc = m;
+                                   e.src = src;
+                                   e.dst = dst;
+                                   if (filter_(e, dst, filter_data_)
+                                       != edge_filter_choice::keep)
+                                     {
+                                       cond = bddfalse;
+                                       return;
+                                     }
+                                 }
                                if (scc_of(dst) != scc
                                    || (m & sets)
                                    || (seen[dst] && !cur[dst]))
